# Project libraries
from . import BaseResource
from .addon import Addon


class LogDrain(BaseResource):
    _strs = ["id", "url", "token"]
    _map = {"addon": Addon}
    _dates = ["created_at", "updated_at"]
    _pks = ["id"]

    def __init__(self):
        self.app = None
        super(LogDrain, self).__init__()

    def __repr__(self):
        return "<logdrain '{0}'>".format(self.id)

    def remove(self):
        r = self._h._http_resource(method="DELETE", resource=("apps", self.app.id, "log-drains", self.id))

        r.raise_for_status()

        return r.ok



# Third party libraries
import requests

# Project libraries
from . import BaseResource


class LogSession(BaseResource):
    _strs = ["id", "logplex_url", "dyno", "source"]
    _ints = ["lines"]
    _bools = ["tail"]
    _dates = ["created_at", "updated_at"]
    _pks = ["id"]

    def __init__(self):
        self.app = None
        super(LogSession, self).__init__()

    def __repr__(self):
        return "<logsession '{0}'>".format(self.id)

    def stream(self, timeout=False):
        r = requests.get(self.logplex_url, verify=False, stream=True, timeout=timeout)
        return r.iter_lines()

    def get(self, timeout=False):
        r = requests.get(self.logplex_url, verify=False, stream=True, timeout=timeout)
        return r.content.decode("utf-8")



# Project libraries
from . import BuildPack, BaseResource  # noqa


class BuildpackInstallation(BaseResource):
    """Heroku Buildpack Insallation."""

    _map = {"buildpack": Buildpack}  # noqa
    _ints = ["ordinal"]
    _pks = ["ordinal"]

    def __repr__(self):
        return "<buildpack-installation '{}'>".format(self.buildpack)



# -*- coding: utf-8 -*-

"""
heroku3.structures
~~~~~~~~~~~~~~~~~

This module contains the specific Heroku.py data types.
"""


class KeyedListResource(object):
    """docstring for ListResource"""

    def __init__(self, items=None):
        super(KeyedListResource, self).__init__()

        self._h = None
        self._items = items or list()
        self._obj = None
        self._kwargs = {}

    def __len__(self):
        return len(self._items)

    def __repr__(self):
        return repr(self._items)

    def __iter__(self):
        for item in self._items:
            yield item

    def __getitem__(self, key):

        # Support index operators.
        if isinstance(key, int):
            if abs(key) <= len(self._items):
                return self._items[key]

        v = self.get(key)

        if v is None:
            raise KeyError(key)

        return v

    def __contains__(self, key):

        v = None
        try:
            v = self[key]
        except KeyError:
            return False
        else:
            if v is None:
                return False
            else:
                return True

    def add(self, *args, **kwargs):

        try:
            return self[0].new(*args, **kwargs)
        except IndexError:
            o = self._obj()
            o._h = self._h
            o.__dict__.update(self._kwargs)

            return o.new(*args, **kwargs)

    def remove(self, key):
        if hasattr(self[0], "delete"):
            return self[key].delete()

    def get(self, key):
        for item in self:
            if key in item._ids:
                return item

    def __delitem__(self, key):
        self[key].delete()

    def append(self, items):
        self._items.append(items)

    def change_connection(self, h):
        for item in self._items:
            item.change_connection(h)

        return self


class DynoListResource(KeyedListResource):
    """KeyedListResource with basic filtering for process types."""

    def __init__(self, *args, **kwargs):
        super(DynoListResource, self).__init__(*args, **kwargs)

    def __getitem__(self, key):

        try:
            return super(DynoListResource, self).__getitem__(key)
        except KeyError as why:

            c = [p for p in self._items if key == p.type]

            if c:
                return DynoTypeListResource(items=c)
            else:
                raise why


class DynoTypeListResource(DynoListResource):
    """KeyedListResource with basic filtering for process types."""

    def __init__(self, *args, **kwargs):

        super(DynoTypeListResource, self).__init__(*args, **kwargs)


class SSHKeyListResource(KeyedListResource):
    """KeyedListResource with clearing for ssh keys."""

    def __init__(self, *args, **kwargs):

        super(SSHKeyListResource, self).__init__(*args, **kwargs)


class FilteredListResource(KeyedListResource):
    filter_func = staticmethod(lambda item: True)

    def __init__(self, items=None):
        items = [item for item in items if self.filter_func(item)] if items else []
        super(FilteredListResource, self).__init__(items)


def filtered_key_list_resource_factory(filter_func):
    return type("FilteredListResource", (FilteredListResource,), {"filter_func": staticmethod(filter_func)})



# -*- coding: utf-8 -*-

"""
heroku3.helpers
~~~~~~~~~~~~~~

This module contians the helpers.
"""

import sys

# Third party libraries
from dateutil.parser import parse as parse_datetime

if sys.version_info > (3, 0):
    basestring = (str, bytes)


def is_collection(obj):
    """Tests if an object is a collection.

    Returns:
      bool: whether `obj` is considered a collection (returns True) or not
          (returns False)
    """
    col = getattr(obj, "__getitem__", False)
    val = False if (not col) else True

    if isinstance(obj, basestring):
        val = False

    return val


# from kennethreitz/python-github3
def to_python(
    obj, in_dict, strs=None, dates=None, ints=None, objects=None, arrays=None, bools=None, dicts=None, **kwargs
):
    """Extends a given object for API Consumption.

    :param obj: Object to extend.
    :param in_dict: Dict to extract data from.
    :param string_keys: List of in_dict keys that will be extracted as strings.
    :param date_keys: List of in_dict keys that will be extrad as datetimes.
    :param object_map: Dict of {key, obj} map, for nested object results.
    """
    d = dict()

    if strs:
        for in_key in strs:
            d[in_key] = in_dict.get(in_key)

    if dates:
        for in_key in dates:
            in_date = in_dict.get(in_key)
            if in_date is not None:
                try:
                    out_date = parse_datetime(in_date)
                except TypeError as e:
                    raise e

                d[in_key] = out_date
            else:
                d[in_key] = None

    if ints:
        for in_key in ints:
            if (in_dict is not None) and (in_dict.get(in_key) is not None):
                d[in_key] = int(in_dict.get(in_key))

    if bools:
        for in_key in bools:
            if in_dict.get(in_key) is not None:
                d[in_key] = bool(in_dict.get(in_key))

    if dicts:
        for in_key in dicts:
            if in_dict.get(in_key) is not None:
                d[in_key] = dict(in_dict.get(in_key))
    if objects:
        for (k, v) in objects.items():
            if in_dict.get(k):
                d[k] = v.new_from_dict(in_dict.get(k))

    if arrays:
        for (k, v) in arrays.items():
            if in_dict.get(k):
                d[k] = [v.new_from_dict(i) for i in in_dict.get(k)]

    obj.__dict__.update(d)
    obj.__dict__.update(kwargs)

    # Save the dictionary, for write comparisons.
    # obj._cache = d
    # obj.__cache = in_dict

    return obj



# Project libraries
from . import BaseResource


class Key(BaseResource):
    """Heroku SSH Key."""

    _strs = ["id", "public_key", "email", "fingerprint", "comment"]
    _dates = ["created_at", "updated_at"]
    _pks = ["id"]

    def __init__(self):
        super(Key, self).__init__()

    def __repr__(self):
        return "<key '{0}-{1}'>".format(self.id, self.email)

    def delete(self):
        """Deletes the key."""
        r = self._h._http_resource(method="DELETE", resource=("account", "keys", self.id))

        r.raise_for_status()
        return self



# Project libraries
from . import User, BaseResource


class Grant(BaseResource):
    """Heroku Grant"""

    _strs = ["code", "id", "type"]
    _ints = ["expires_in"]
    _pks = ["id"]

    def __init__(self):
        # self.app = None
        super(Grant, self).__init__()

    def __repr__(self):
        return "<grant '{0} {1} {2}'>".format(self.id, self.code, self.expires_in)


class RefreshToken(BaseResource):
    """Heroku RefreshToken"""

    _strs = ["token", "id"]
    _ints = ["expires_in"]
    _pks = ["id"]

    def __init__(self):
        # self.app = None
        super(RefreshToken, self).__init__()

    def __repr__(self):
        return "<refreshtoken '{0} {1} {2}'>".format(self.id, self.token, self.expires_in)


class AccessToken(BaseResource):
    """Heroku AccessToken"""

    _strs = ["token", "id"]
    _ints = ["expires_in"]
    _pks = ["id"]

    def __init__(self):
        # self.app = None
        super(AccessToken, self).__init__()

    def __repr__(self):
        return "<AccessToken '{0} {1} {2}'>".format(self.id, self.token, self.expires_in)


# class Client(BaseResource):
# """Heroku Client"""
#
# _strs = ['id', 'name', 'redirect_uri']
# _pks = ['id']
#
# def __init__(self):
# self.app = None
# super(Client, self).__init__()
#
# def __repr__(self):
# return "<Client '{0} {1} {2}'>".format(self.id, self.name, self.redirect_uri)


class OAuthClient(BaseResource):
    """Heroku OAuthClient"""

    _strs = ["id", "name", "redirect_uri", "secret"]
    _dates = ["created_at", "updated_at"]
    _bools = ["ignores_delinquent"]
    _pks = ["id"]

    def __init__(self):
        # self.app = None
        super(OAuthClient, self).__init__()

    def __repr__(self):
        return "<OAuthClient '{0} {1} {2} {3}'>".format(self.id, self.name, self.redirect_uri, self.created_at)

    def delete(self):
        """
        Destroys the current OAuthClient
        """
        r = self._h._http_resource(method="DELETE", resource=("oauth", "clients", self.id))
        r.raise_for_status()
        return r.ok

    def update(self, name=None, redirect_uri=None):

        assert name or redirect_uri
        payload = {"name": name, "redirect_uri": redirect_uri}
        r = self._h._http_resource(
            method="PATCH", resource=("oauth", "clients", self.id), data=self._h._resource_serialize(payload)
        )

        r.raise_for_status()
        item = self._h._resource_deserialize(r.content.decode("utf-8"))
        return OAuthClient.new_from_dict(item, h=self._h)


class OAuthAuthorization(BaseResource):
    _strs = ["id"]
    _dates = ["created_at", "updated_at"]
    _map = {"grant": Grant, "refresh_token": RefreshToken, "access_token": AccessToken, "client": OAuthClient}
    _pks = ["id"]
    order_by = "id"

    def __init__(self):
        # self.app = None
        super(OAuthAuthorization, self).__init__()

    def __repr__(self):
        return "<OAuthAuthorization '{0} {1} {2}'>\n".format(self.id, self.created_at, self.client.id)

    def delete(self):
        """
        Destroys the current OAuthAuthorization
        """
        r = self._h._http_resource(method="DELETE", resource=("oauth", "authorizations", self.id))
        r.raise_for_status()
        return r.ok


class OAuthToken(BaseResource):
    _strs = ["id"]
    _dates = ["created_at", "updated_at"]
    _map = {
        "grant": Grant,
        "refresh_token": RefreshToken,
        "access_token": AccessToken,
        "client": OAuthClient,
        "authorization": OAuthAuthorization,
        "user": User,
    }
    _pks = ["id"]
    order_by = "id"

    def __init__(self):
        # self.app = None
        super(OAuthToken, self).__init__()

    def __repr__(self):
        return "<OAuthToken '{0} {1} {2} {3}'>\n".format(self.id, self.created_at, self.user.id, self.grant.type)


class Session(BaseResource):
    _strs = ["id"]
    _pks = ["id"]
    order_by = "id"

    def __init__(self):
        # self.app = None
        super(Session, self).__init__()

    def __repr__(self):
        return "<Session '{0}'>\n".format(self.id)
